@@ -1,6 +1,6 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999-2002,2006,2010 Rami Lehti,Pablo Virolainen,
+ * Copyright (C) 1999-2002,2006,2010,2016 Rami Lehti,Pablo Virolainen,
  * Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
@@ -21,29 +21,34 @@
 
 #ifndef _GEN_LIST_H_INCLUDED
 #define _GEN_LIST_H_INCLUDED
+#include <pcre.h>
 #include "seltree.h"
 #include "list.h"
-#include "gnu_regex.h"
+
+#define RESTRICTION_TYPE unsigned int
+#define RESTRICTION_FT_REG   (1U<<0) /* file */
+#define RESTRICTION_FT_DIR   (1U<<1) /* dir */
+#define RESTRICTION_FT_FIFO  (1U<<2) /* fifo */
+#define RESTRICTION_FT_LNK   (1U<<3) /* link */
+#define RESTRICTION_FT_BLK   (1U<<4) /* block device */
+#define RESTRICTION_FT_CHR   (1U<<5) /* char device */
+#define RESTRICTION_FT_SOCK  (1U<<6) /* socket */
+#define RESTRICTION_FT_DOOR  (1U<<7) /* door */
+#define RESTRICTION_FT_PORT  (1U<<8) /* port */
+#define RESTRICTION_NULL 0U
 
 /* DB_FOO are anded together to form rx_rule's attr */
 
 typedef struct rx_rule {
   char* rx; /* Regular expression in text form */
-  regex_t* crx; /* Compiled regexp */
+  pcre* crx; /* Compiled regexp */
   DB_ATTR_TYPE attr; /* Which attributes to save */
   long  conf_lineno; /* line no. of rule definition*/
+  RESTRICTION_TYPE restriction;
 } rx_rule;
 
 int compare_node_by_path(const void *n1, const void *n2);
 
-/*
- * gen_list()
- * Generates the file list
- * from the lists of rx_rule's
- */
-
-list* gen_list(list* prxlist,list* nrxlist,list* erxlist);
-
 /* 
  * gen_tree()
  * Generates the file tree
@@ -65,11 +70,9 @@ void populate_tree(seltree* tree);
 
 char* strrxtok(char*);
 
-int check_list_for_match(list*,char*,DB_ATTR_TYPE*);
+int check_rxtree(char* filename,seltree* tree, DB_ATTR_TYPE* attr, mode_t perm);
 
-int check_rxtree(char* filename,seltree* tree, DB_ATTR_TYPE* attr);
-
-db_line* get_file_attrs(char* filename,DB_ATTR_TYPE attr);
+db_line* get_file_attrs(char* filename,DB_ATTR_TYPE attr, struct AIDE_STAT_TYPE *fs);
 
 seltree* get_seltree_node(seltree* tree,char* path);
 
