@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999-2006,2010 Rami Lehti, Pablo Virolainen, Richard
- * van den Berg, Hannes von Haugwitz
+ * Copyright (C) 1999-2006,2010,2011,2013,2015,2016 Rami Lehti, Pablo
+ * Virolainen, Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -34,12 +34,15 @@
 #include "db_config.h"
 #include "gen_list.h"
 #include "symboltable.h"
+#include "md.h"
 #include "util.h"
 #include "base64.h"
 /*for locale support*/
 #include "locale-aide.h"
 /*for locale support*/
+#ifdef WITH_CURL
 #include "fopen.h"
+#endif
 
 #define BUFSIZE 4096
 #define ZBUFSIZE 16384
@@ -96,27 +99,8 @@ int commandconf(const char mode,const char* line)
   case 'D': {
     /* Let's do it */
     int rv=-1;
-    char* new_config=NULL;
-    char* homedir=NULL;
 
-    /* support for ~ in the config file string 
-       ~ must be the first character and it will be
-       replaced with HOME-environment variable
-     */
-    if(config[0]=='~'){
-      if((homedir=getenv("HOME"))){
-	new_config=(char*)malloc(sizeof(char)*
-				 (strlen(config)+strlen(homedir)+1));
-	memcpy(new_config,homedir,strlen(homedir));
-	memcpy(new_config+strlen(homedir),
-	       config+sizeof(char),strlen(config+sizeof(char)));
-	l=(strlen(config)+strlen(homedir));
-	new_config[l]='\0';
-	free(config);
-	config=new_config;
-	/* Don't free(homedir); because it is not safe on some platforms */
-      }
-    }
+    config = expand_tilde(config);
     if (config!=NULL && strcmp(config,"-")==0) {
       error(255,_("Config from stdin\n"));
       rv=0;
@@ -124,7 +108,7 @@ int commandconf(const char mode,const char* line)
       
       rv=access(config,R_OK);
       if(rv==-1){
-	error(0,_("Cannot access config file:%s:%s\n"),config,strerror(errno));
+	error(0,_("Cannot access config file: %s: %s\n"),config,strerror(errno));
       }
     }
     
@@ -243,7 +227,9 @@ int db_input_wrapper(char* buf, int max_size, int db)
   int c=0;
   int err=0;
   int* domd=NULL;
+#ifdef WITH_CURL
   url_t* db_url=NULL;
+#endif
 #ifdef WITH_MHASH
   char* tmp=NULL;
   MHASH* md=NULL;
@@ -254,9 +240,12 @@ int db_input_wrapper(char* buf, int max_size, int db)
 #ifdef WITH_ZLIB
   gzFile* db_gzp=NULL;
 #endif
+  struct md_container *mdc;
   switch(db) {
   case DB_OLD: {
+#ifdef WITH_CURL
     db_url=conf->db_in_url;
+#endif
     
     domd=&(conf->do_dboldmd);
 #ifdef WITH_MHASH
@@ -271,7 +260,9 @@ int db_input_wrapper(char* buf, int max_size, int db)
     break;
   }
   case DB_NEW: {
+#ifdef WITH_CURL
     db_url=conf->db_new_url;
+#endif
     
     domd=&(conf->do_dbnewmd);
 #ifdef WITH_MHASH
@@ -293,6 +284,9 @@ int db_input_wrapper(char* buf, int max_size, int db)
   case url_https:
   case url_ftp: {
     retval=url_fread(buf,1,max_size,(URL_FILE *)*db_filep);
+    if ((mdc = (db == DB_OLD ? conf->mdc_in : conf->mdc_out))) {
+        update_md(mdc, buf, retval);
+    }
     break;
   } 
   default:
@@ -365,6 +359,10 @@ int db_input_wrapper(char* buf, int max_size, int db)
 #endif /* WITH_MHASH */ 
 #endif /* WITH_ZLIB */
 
+  if ((mdc = (db == DB_OLD ? conf->mdc_in : conf->mdc_out))) {
+      update_md(mdc, buf, retval);
+  }
+
 #ifdef WITH_MHASH    
   if(*domd){
     if(!*md){
@@ -642,7 +640,7 @@ int handle_endif(int doit,int allow_else){
 	    return 0;
 	  }
 	  else {
-	    conferror("Ambigous else");
+	    conferror("Ambiguous else");
 	    return -1;
 	  }
 	}
@@ -716,7 +714,7 @@ int do_ifxhost(int mode,char* name)
   return (handle_endif(doit,1));
 }
 
-list* append_rxlist(char* rx,DB_ATTR_TYPE attr,list* rxlst)
+list* append_rxlist(char* rx,DB_ATTR_TYPE attr,list* rxlst, RESTRICTION_TYPE restriction)
 {
   extern long conf_lineno; /* defined & set in conf_lex.l */
     
@@ -725,6 +723,7 @@ list* append_rxlist(char* rx,DB_ATTR_TYPE attr,list* rxlst)
   r->rx=rx;
   r->attr=attr;
   r->conf_lineno = conf_lineno;
+  r->restriction = restriction;
   if (attr&DB_CHECKINODE && attr&DB_CTIME)
     error(20,"Rule at line %li has c and I flags enabled at the same time. If same inode is found, flag c is ignored\n",conf_lineno);
   update_db_out_order(r->attr);
@@ -738,7 +737,16 @@ void do_groupdef(char* group,DB_ATTR_TYPE value)
   list* r=NULL;
   symba* s=NULL;
 
+  if (strcmp(group, "ignore_list") == 0) {
+      error(2, "DEPRECATION WARNING, Using '%s' is deprecated. Update your config and use '%s' instead.\n", group, "report_ignore_changed_attrs");
+      group = "report_ignore_changed_attrs";
+  } else if (strcmp(group, "report_attributes") == 0) {
+      error(2, "DEPRECATION WARNING, Using '%s' is deprecated. Update your config and use '%s' instead.\n", group, "report_force_attrs");
+      group = "report_force_attrs";
+  }
+
   if((r=list_find(group,conf->groupsyms))){
+      error(2, "Warning: group '%s' is redefined\n", group);
       ((symba*)r->data)->ival=value;
       return;
   }
@@ -749,6 +757,19 @@ void do_groupdef(char* group,DB_ATTR_TYPE value)
   conf->groupsyms=list_append(conf->groupsyms,(void*)s);
 }
 
+RESTRICTION_TYPE get_restrictionval(char* ch) {
+    if (strcmp(ch, "f") == 0) { return RESTRICTION_FT_REG; }
+    else if (strcmp(ch, "d") == 0) { return RESTRICTION_FT_DIR; }
+    else if (strcmp(ch, "p") == 0) { return RESTRICTION_FT_FIFO; }
+    else if (strcmp(ch, "l") == 0) { return RESTRICTION_FT_LNK; }
+    else if (strcmp(ch, "b") == 0) { return RESTRICTION_FT_BLK; }
+    else if (strcmp(ch, "c") == 0) { return RESTRICTION_FT_CHR; }
+    else if (strcmp(ch, "s") == 0) { return RESTRICTION_FT_SOCK; }
+    else if (strcmp(ch, "D") == 0) { return RESTRICTION_FT_DOOR; }
+    else if (strcmp(ch, "P") == 0) { return RESTRICTION_FT_PORT; }
+    else { return RESTRICTION_NULL; }
+}
+
 DB_ATTR_TYPE get_groupval(char* group)
 {
   list* r=NULL;
@@ -896,6 +917,77 @@ void do_verbdef(char* val)
   }
 }
 
+void do_rootprefix(char* val) {
+    if (conf->root_prefix_length == 0) {
+        conf->root_prefix=val;
+        conf->root_prefix_length=strlen(conf->root_prefix);
+        if (conf->root_prefix_length && conf->root_prefix[conf->root_prefix_length-1] == '/') {
+            conf->root_prefix[--conf->root_prefix_length] = '\0';
+            error(200,_("Removed trailing '/' from root prefix \n"));
+        }
+        error(200,_("Root prefix set to '%s'\n"), conf->root_prefix);
+    } else {
+        error(200,_("Root prefix already set to '%s'\n"), conf->root_prefix);
+    }
+}
+
+#ifdef WITH_E2FSATTRS
+#define easy_e2fsattrs_case(c,f) \
+case c: { \
+    conf->report_ignore_e2fsattrs|=f; \
+    break; \
+}
+
+void do_report_ignore_e2fsattrs(char* val) {
+    conf->report_ignore_e2fsattrs = 0UL;
+    while (*val) {
+        switch(*val){
+            /* source for mappings see compare_db.c */
+            easy_e2fsattrs_case('s',EXT2_SECRM_FL)
+            easy_e2fsattrs_case('u',EXT2_UNRM_FL)
+            easy_e2fsattrs_case('S',EXT2_SYNC_FL)
+            easy_e2fsattrs_case('D',EXT2_DIRSYNC_FL)
+            easy_e2fsattrs_case('i',EXT2_IMMUTABLE_FL)
+            easy_e2fsattrs_case('a',EXT2_APPEND_FL)
+            easy_e2fsattrs_case('d',EXT2_NODUMP_FL)
+            easy_e2fsattrs_case('A',EXT2_NOATIME_FL)
+            easy_e2fsattrs_case('c',EXT2_COMPR_FL)
+            easy_e2fsattrs_case('B',EXT2_COMPRBLK_FL)
+            easy_e2fsattrs_case('Z',EXT2_DIRTY_FL)
+            easy_e2fsattrs_case('X',EXT2_NOCOMPR_FL)
+#ifdef EXT2_ECOMPR_FL
+            easy_e2fsattrs_case('E',EXT2_ECOMPR_FL)
+#else
+            easy_e2fsattrs_case('E',EXT4_ENCRYPT_FL)
+#endif
+            easy_e2fsattrs_case('j',EXT3_JOURNAL_DATA_FL)
+            easy_e2fsattrs_case('I',EXT2_INDEX_FL)
+            easy_e2fsattrs_case('t',EXT2_NOTAIL_FL)
+            easy_e2fsattrs_case('T',EXT2_TOPDIR_FL)
+#ifdef EXT4_EXTENTS_FL
+            easy_e2fsattrs_case('e',EXT4_EXTENTS_FL)
+#endif
+#ifdef EXT4_HUGE_FILE_FL
+            easy_e2fsattrs_case('h',EXT4_HUGE_FILE_FL)
+#endif
+#ifdef FS_NOCOW_FL
+            easy_e2fsattrs_case('C',FS_NOCOW_FL)
+#endif
+#ifdef EXT4_INLINE_DATA_FL
+            easy_e2fsattrs_case('N',EXT4_INLINE_DATA_FL)
+#endif
+            case '0': {
+                 break;
+            }
+            default: {
+                 error(0,_("Ignore invalid ext2 file attribute: '%c'\n"),*val);
+                 break;
+            }
+        }
+        *val++;
+    }
+}
+#endif
 
 const char* aide_key_7=CONFHMACKEY_07;
 const char* db_key_7=DBHMACKEY_07;
