@@ -1,8 +1,9 @@
-/* A Bison parser, made by GNU Bison 3.0.4.  */
+/* A Bison parser, made by GNU Bison 3.3.2.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2019 Free Software Foundation,
+   Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,6 +31,9 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+/* Undocumented macros, especially those whose name start with YY_,
+   are private implementation details.  Do not rely on them.  */
+
 #ifndef YY_CONF_SRC_CONF_YACC_H_INCLUDED
 # define YY_CONF_SRC_CONF_YACC_H_INCLUDED
 /* Debug traces.  */
@@ -204,13 +208,13 @@ extern int confdebug;
 
 union YYSTYPE
 {
-#line 45 "src/conf_yacc.y" /* yacc.c:1909  */
+#line 45 "src/conf_yacc.y" /* yacc.c:1921  */
 
   char* s;
   DB_ATTR_TYPE i;
   RESTRICTION_TYPE r;
 
-#line 214 "src/conf_yacc.h" /* yacc.c:1909  */
+#line 218 "src/conf_yacc.h" /* yacc.c:1921  */
 };
 
 typedef union YYSTYPE YYSTYPE;
