@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999-2006,2010 Rami Lehti, Pablo Virolainen, Richard
- * van den Berg, Hannes von Haugwitz
+ * Copyright (C) 1999-2006,2010,2011,2013 Rami Lehti, Pablo Virolainen,
+ * Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -26,6 +26,11 @@
 #include "db.h"
 #include "db_file.h"
 #include "db_disk.h"
+#include "md.h"
+
+#ifdef WITH_CURL
+#include "fopen.h"
+#endif
 
 #ifdef WITH_PSQL
 #include "db_sql.h"
@@ -131,6 +136,47 @@ const char* db_namealias[db_alias_size] = {
 const int db_aliasvalue[db_alias_size] = {
   db_lnkcount } ;       /* "count",  */
 
+static struct md_container *init_db_attrs(URL_TYPE type) {
+    struct md_container *mdc = NULL;
+    if (conf->db_attrs) {
+        switch (type) {
+            case url_stdout:
+            case url_stderr:
+            case url_fd:
+            case url_file:
+            #ifdef WITH_CURL
+            case url_http:
+            case url_https:
+            case url_ftp:
+            #endif /* WITH CURL */
+                mdc = malloc(sizeof(struct md_container)); /* freed in close_db_attrs */
+                mdc->todo_attr = conf->db_attrs;
+                init_md(mdc);
+                break;
+            #ifdef WITH_PSQL
+            case url_sql:
+                break;
+            #endif /* WITH_PSQL */
+            default :
+                error(200,_("init_db_attrs(): Unknown url type.\n"));
+        }
+    }
+    return mdc;
+}
+
+static db_line *close_db_attrs (struct md_container *mdc, char *url_value) {
+    db_line *line = NULL;
+    if (mdc != NULL) {
+        close_md(mdc);
+        line = malloc(sizeof(struct db_line));
+        line->filename = url_value;
+        line->perm = 0;
+        line->attr = conf->db_attrs;
+        md2line(mdc, line);
+        free(mdc);
+    }
+    return line;
+}
 
 int db_init(int db)
 {
@@ -149,6 +195,7 @@ int db_init(int db)
 
 
   case DB_OLD: {
+    conf->mdc_in = init_db_attrs((conf->db_in_url)->type);
     rv=be_init(1,conf->db_in_url,0);
     if(rv==NULL) {
       error(200,_("db_in is null\n"));      
@@ -160,6 +207,7 @@ int db_init(int db)
   }
   case DB_WRITE: {    
 #ifdef WITH_ZLIB
+    conf->mdc_out = init_db_attrs((conf->db_out_url)->type);
     if(conf->gzip_dbout){
        rv=be_init(0,conf->db_out_url,conf->gzip_dbout);
        conf->db_gzout=rv;
@@ -180,6 +228,7 @@ int db_init(int db)
     return RETOK;
   }
   case DB_NEW: {
+    conf->mdc_out = init_db_attrs((conf->db_new_url)->type);
     rv=be_init(1,conf->db_new_url,0);
     if(rv==NULL) {
       error(200,_("db_new is null\n"));      
@@ -206,7 +255,7 @@ db_line* db_readline(int db){
     /*
       Nothing else to be done?
      */
-    s=db_readline_disk(db);
+    s=db_readline_disk();
     return s;
   }
   
@@ -323,22 +372,16 @@ db_line* db_char2line(char** ss,int db){
 
   int i;
   db_line* line=(db_line*)malloc(sizeof(db_line)*1);
-  url_t* db_url=NULL;
-  FILE** db_filep=NULL;
   int* db_osize=0;
   DB_FIELD** db_order=NULL;
 
   switch (db) {
   case DB_OLD: {
-    db_url=conf->db_in_url;
-    db_filep=&(conf->db_in);
     db_osize=&(conf->db_in_size);
     db_order=&(conf->db_in_order);
     break;
   }
   case DB_NEW: {
-    db_url=conf->db_new_url;
-    db_filep=&(conf->db_new);
     db_osize=&(conf->db_new_size);
     db_order=&(conf->db_new_order);
     break;
@@ -370,6 +413,7 @@ db_line* db_char2line(char** ss,int db){
   line->bcount=0;
   line->size=0;
   line->filename=NULL;
+  line->fullpath=NULL;
   line->linkname=NULL;
   line->acl=NULL;
   line->xattrs=NULL;
@@ -383,7 +427,8 @@ db_line* db_char2line(char** ss,int db){
     case db_filename : {
       if(ss[(*db_order)[i]]!=NULL){
 	decode_string(ss[(*db_order)[i]]);
-	line->filename=strdup(ss[(*db_order)[i]]);
+	line->fullpath=strdup(ss[(*db_order)[i]]);
+	line->filename=line->fullpath;
       } else {
 	error(0,"db_char2line():Error while reading database\n");
 	exit(EXIT_FAILURE);
@@ -764,25 +809,19 @@ int db_writeline(db_line* line,db_config* dbconf){
   return RETFAIL;
 }
 
-int db_close(db_config* dbconf)
-{
-  if (dbconf==NULL) return RETOK;
-  
-  switch (dbconf->db_out_url->type) {
+void db_close() {
+  switch (conf->db_out_url->type) {
   case url_stdout:
   case url_stderr:
   case url_fd:
   case url_file: {
     if (
 #ifdef WITH_ZLIB
-       (dbconf->gzip_dbout && dbconf->db_gzout) ||
+       (conf->gzip_dbout && conf->db_gzout) ||
 #endif
-       (dbconf->db_out!=NULL)) {
-      if (db_close_file(dbconf)==RETOK) {
-	return RETOK;
-      }
+       (conf->db_out!=NULL)) {
+        db_close_file(conf);
     }
-    return RETFAIL;
     break;
   }
 #ifdef WITH_CURL
@@ -790,29 +829,27 @@ int db_close(db_config* dbconf)
   case url_https:
   case url_ftp:
     {
-      url_fclose(dbconf->db_out);
+        if (conf->db_out!=NULL) {
+            url_fclose(conf->db_out);
+        }
       break;
     }
 #endif /* WITH CURL */
 #ifdef WITH_PSQL
   case url_sql: {
-    if (dbconf->db_out!=NULL) {
-      if (db_close_sql(dbconf->db_out)==RETOK) {
-	return RETOK;
-      } else {
-	return RETFAIL;
-      }
+    if (conf->db_out!=NULL) {
+      db_close_sql(conf->db_out);
     }
-    return RETOK;
     break;
   }
 #endif
   default : {
     error(0,_("db_close():Unknown output in db out.\n"));    
-    return RETFAIL;
   } 
   }
-  return RETFAIL;
+  conf->line_db_in = close_db_attrs(conf->mdc_in, (conf->db_in_url)->value);
+  conf->line_db_out = close_db_attrs(conf->mdc_out, (conf->action&DO_DIFF
+          ? conf->db_new_url : conf->db_out_url)->value);
 }
 
 void free_db_line(db_line* dl)
@@ -827,7 +864,8 @@ void free_db_line(db_line* dl)
   checked_free(dl->sha1);
   checked_free(dl->rmd160);
   checked_free(dl->tiger);
-  checked_free(dl->filename);
+  dl->filename=NULL;
+  checked_free(dl->fullpath);
   checked_free(dl->linkname);
   
 #ifdef WITH_MHASH
