@@ -1,7 +1,7 @@
 /* aide, Advanced Intrusion Detection Environment
  *
- * Copyright (C) 1999-2007,2010 Rami Lehti, Pablo Virolainen, Mike
- * Markley, Richard van den Berg, Hannes von Haugwitz
+ * Copyright (C) 1999-2007,2010-2013,2016,2018 Rami Lehti, Pablo Virolainen,
+ * Mike Markley, Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -32,9 +32,9 @@
 #include "types.h"
 #include "base64.h"
 #include "db_file.h"
+#include "gen_list.h"
 #include "conf_yacc.h"
 #include "util.h"
-#include "db_sql.h" /* typedefs */
 #include "commandconf.h"
 /*for locale support*/
 #include "locale-aide.h"
@@ -85,7 +85,7 @@ void handle_gzipped_input(int out,gzFile* gzp){
       }
       if (write(out, buf,nread) != tmp)
       {
-        error(0,_("write() failed: %m\n"));
+        error(0,_("write() failed: %s\n"), strerror(errno));
         exit(1);
       }
       
@@ -142,6 +142,10 @@ int dofprintf( const char* s,...)
   retval=vsnprintf(temp,retval+1,s,ap);
   va_end(ap);
   
+  if (conf->mdc_out) {
+      update_md(conf->mdc_out,temp ,retval);
+  }
+
 #ifdef WITH_MHASH
   if(conf->do_dbnewmd)
     mhash(conf->dbnewmd,(void*)temp,retval);
@@ -169,33 +173,18 @@ int db_file_read_spec(int db){
   int i=0;
   int* db_osize=0;
   DB_FIELD** db_order=NULL;
-  FILE** db_filep=NULL;
-  url_t* db_url=NULL;
-#ifdef WITH_ZLIB
-  gzFile* db_gzp=NULL;
-#endif
 
   switch (db) {
   case DB_OLD: {
     db_osize=&(conf->db_in_size);
     db_order=&(conf->db_in_order);
-    db_filep=&(conf->db_in);
-    db_url=conf->db_in_url;
     db_lineno=&db_in_lineno;
-#ifdef WITH_ZLIB
-    db_gzp=&(conf->db_gzin);
-#endif
     break;
   }
   case DB_NEW: {
     db_osize=&(conf->db_new_size);
     db_order=&(conf->db_new_order);
-    db_filep=&(conf->db_new);
-    db_url=conf->db_new_url;
     db_lineno=&db_new_lineno;
-#ifdef WITH_ZLIB
-    db_gzp=&(conf->db_gznew);
-#endif
     break;
   }
   }
@@ -281,7 +270,7 @@ int db_file_read_spec(int db){
   }
   if (conf->attr==DB_ATTR_UNDEF) {
     conf->attr=0;
-    error(0,"Database does not have attr field.\nComparation may be incorrect\nGenerating attr-field from dbspec\nIt might be a good Idea to regenerate databases. Sorry.\n");
+    error(0,"Database does not have attr field.\nComparison may be incorrect\nGenerating attr-field from dbspec\nIt might be a good Idea to regenerate databases. Sorry.\n");
     for(i=0;i<conf->db_in_size;i++) {
       conf->attr|=1<<(*db_order)[i];
     }
@@ -308,9 +297,6 @@ char** db_readline_file(int db){
   DB_FIELD** db_order=NULL;
   FILE** db_filep=NULL;
   url_t* db_url=NULL;
-#ifdef WITH_ZLIB
-  gzFile* db_gzp=NULL;
-#endif
 
   switch (db) {
   case DB_OLD: {
@@ -325,10 +311,6 @@ char** db_readline_file(int db){
     db_filep=&(conf->db_in);
     db_url=conf->db_in_url;
     db_lineno=&db_in_lineno;
-    
-#ifdef WITH_ZLIB
-    db_gzp=&(conf->db_gzin);
-#endif
     break;
   }
   case DB_NEW: {
@@ -343,9 +325,6 @@ char** db_readline_file(int db){
     db_filep=&(conf->db_new);
     db_url=conf->db_new_url;
     db_lineno=&db_new_lineno;
-#ifdef WITH_ZLIB
-    db_gzp=&(conf->db_gznew);
-#endif
     break;
   }
   }
@@ -354,18 +333,16 @@ char** db_readline_file(int db){
     db_buff(db,*db_filep);
     
     token=db_scan();
-    while((token!=TDBSPEC)){
+    while((token!=TDBSPEC && token!=TEOF)){
 
       switch(token){
       case TUNKNOWN: {
 	continue;
-	break;
       }
       case TBEGIN_DB: {
 	token=db_scan();
 	gotbegin_db=1;
 	continue;
-	break;
       }
       case TNEWLINE: {
 	if(gotbegin_db){
@@ -380,7 +357,6 @@ char** db_readline_file(int db){
       case TGZIPHEADER: {
 	error(0,"Gzipheader found inside uncompressed db!\n");
 	return NULL;
-	break;
       }
       default: {
 	/* If it is anything else we quit */
@@ -389,11 +365,6 @@ char** db_readline_file(int db){
 	break;
       }
       }
-      if(token==TEOF){
-	break;
-      }
-
-      token=db_scan();
     }
 
     if(FORCEDBMD&&!gotbegin_db){
@@ -674,7 +645,7 @@ int db_writelong(AIDE_OFF_TYPE i,FILE* file,int a)
     dofprintf(" ");
   }
   
-#if SIZEOF_OFF64_T == SIZEOF_LONG_LONG
+#if defined HAVE_OFF64_TYPE && SIZEOF_OFF64_T == SIZEOF_LONG_LONG || !defined HAVE_OFF64_TYPE && SIZEOF_OFF_T == SIZEOF_LONG_LONG
   return dofprintf("%lli",(long long)i);
 #else
   return dofprintf("%li",i);
@@ -702,7 +673,7 @@ int db_write_byte_base64(byte*data,size_t len,FILE* file,int i,
   }
 
   if(tmpstr){
-    retval=dofprintf(tmpstr);
+    retval=dofprintf("%s", tmpstr);
     free(tmpstr);
     return retval;
   }else {
@@ -741,7 +712,7 @@ int db_write_time_base64(time_t i,FILE* file,int a)
 
 
   tmpstr=encode_base64((byte *)ptr,strlen(ptr));
-  retval=dofprintf(tmpstr);
+  retval=dofprintf("%s", tmpstr);
   free(tmpstr);
   free(ptr);
 
@@ -796,15 +767,17 @@ int db_writespec_file(db_config* dbconf)
   
 #endif
 
-  retval=dofprintf(
-		 "# This file was generated by Aide, version %s\n"
-		 "# Time of generation was %.4u-%.2u-%.2u %.2u:%.2u:%.2u\n",
-		 AIDEVERSION,
-		 st->tm_year+1900, st->tm_mon+1, st->tm_mday,
-		 st->tm_hour, st->tm_min, st->tm_sec
-		 );
-  if(retval==0){
-    return RETFAIL;
+  if(dbconf->database_add_metadata) {
+      retval=dofprintf(
+             "# This file was generated by Aide, version %s\n"
+             "# Time of generation was %.4u-%.2u-%.2u %.2u:%.2u:%.2u\n",
+             AIDEVERSION,
+             st->tm_year+1900, st->tm_mon+1, st->tm_mday,
+             st->tm_hour, st->tm_min, st->tm_sec
+             );
+      if(retval==0){
+        return RETFAIL;
+      }
   }
   if(dbconf->config_version){
     retval=dofprintf(
@@ -836,6 +809,7 @@ int db_writespec_file(db_config* dbconf)
   return RETOK;
 }
 
+#ifdef WITH_ACL
 int db_writeacl(acl_type* acl,FILE* file,int a)
 {
 #ifdef WITH_SUN_ACL
@@ -889,6 +863,7 @@ int db_writeacl(acl_type* acl,FILE* file,int a)
   
   return RETOK;
 }
+#endif
 
 int db_writeline_file(db_line* line,db_config* dbconf, url_t* url){
   int i;
