@@ -1,8 +1,8 @@
 /* aide, Advanced Intrusion Detection Environment
  * vi: ts=8 sw=8
  *
- * Copyright (C) 1999-2002,2004-2006,2009,2010 Rami Lehti, Pablo Virolainen,
- * Mike Markley, Richard van den Berg, Hannes von Haugwitz
+ * Copyright (C) 1999-2002,2004-2006,2009-2011,2013,2018 Rami Lehti, Pablo
+ * Virolainen, Mike Markley, Richard van den Berg, Hannes von Haugwitz
  * $Header$
  *
  * This program is free software; you can redistribute it and/or
@@ -90,8 +90,10 @@ int is_prelinked(int fd) {
         if ((elf = elf_begin (fd, ELF_C_READ, NULL)) == NULL
             || elf_kind(elf) != ELF_K_ELF
             || gelf_getehdr(elf, &ehdr) == NULL
-            || !(ehdr.e_type == ET_DYN || ehdr.e_type == ET_EXEC))
+            || !(ehdr.e_type == ET_DYN || ehdr.e_type == ET_EXEC)) {
+                elf_end(elf);
                 return 0;
+        }
 
         bingo = 0;
         while (!bingo && (scn = elf_nextscn(elf, scn)) != NULL) {
@@ -113,6 +115,7 @@ int is_prelinked(int fd) {
                         }
                 }
         }
+        elf_end(elf);
 
         return bingo;
 }
@@ -154,12 +157,6 @@ pid_t open_prelinked(const char * path, int * fd) {
 
 #endif
 
-void md_init_fail(const char* s,db_line* db,byte** hash,DB_ATTR_TYPE i) {
-  error(0,"Message digest %s initialise failed\nDisabling %s for file %s\n",s,s,db->filename);
-  db->attr=db->attr&(~i);
-  (*hash)=0;
-}
-
 void free_hashes(db_line* dl){
 
 #define free_hash(a) dl->a=NULL
@@ -222,19 +219,19 @@ void calc_md(struct AIDE_STAT_TYPE* old_fs,db_line* line) {
 #endif  
 
 #ifdef HAVE_O_NOATIME
-  filedes=open(line->filename,O_RDONLY|O_NOATIME);
+  filedes=open(line->fullpath,O_RDONLY|O_NOATIME);
   if(filedes<0)
 #endif
-    filedes=open(line->filename,O_RDONLY);
+    filedes=open(line->fullpath,O_RDONLY);
 
   if (filedes==-1) {
     char* er=strerror(errno);
     if (er!=NULL) {
       error(3,"do_md(): open() for %s failed: %s\n",
-	    line->filename,er);
+	    line->fullpath,er);
     } else {
       error(3,"do_md(): open() for %s failed: %i\n",
-	    line->filename,errno);
+	    line->fullpath,errno);
     }
     /*
       Nop. Cannot cal hashes. Mark it.
@@ -266,7 +263,7 @@ void calc_md(struct AIDE_STAT_TYPE* old_fs,db_line* line) {
     pid=0;
     if ( is_prelinked(filedes) ) {
       close(filedes);
-      pid = open_prelinked(line->filename, &filedes);
+      pid = open_prelinked(line->fullpath, &filedes);
       if (pid == 0) {
         error(0, "Error on starting prelink undo\n");
 	return;
@@ -312,7 +309,7 @@ void calc_md(struct AIDE_STAT_TYPE* old_fs,db_line* line) {
 	   r_size-=MMAP_BLOCK_SIZE;
 	 }
 	 if ( buf == MAP_FAILED ) {
-	   error(0,"error mmap'ing %s: %s\n", line->filename,strerror(errno));
+	   error(0,"error mmap'ing %s: %s\n", line->fullpath,strerror(errno));
 	   close(filedes);
 	   close_md(&mdc);
 	   return;
@@ -379,8 +376,8 @@ void calc_md(struct AIDE_STAT_TYPE* old_fs,db_line* line) {
       Something just wasn't correct, so no hash calculated.
     */
     
-    error(5,"File %s was changed so that hash cannot be calculated for it\n"
-	  ,line->filename);
+    error(5,"Entry %s was changed so that hash cannot be calculated for it\n"
+	  ,line->fullpath);
 
     for(i=0;i<db_unknown;i++) {
       if (((1<<i)&stat_diff)!=0) {
@@ -452,6 +449,7 @@ void fs2db_line(struct AIDE_STAT_TYPE* fs,db_line* line) {
   
 }
 
+#ifdef WITH_ACL
 void acl2line(db_line* line) {
   acl_type *ret = NULL;
   
@@ -461,20 +459,20 @@ void acl2line(db_line* line) {
     acl_t acl_d;
     char *tmp = NULL;
 
-    acl_a = acl_get_file(line->filename, ACL_TYPE_ACCESS);
-    acl_d = acl_get_file(line->filename, ACL_TYPE_DEFAULT);
+    acl_a = acl_get_file(line->fullpath, ACL_TYPE_ACCESS);
+    acl_d = acl_get_file(line->fullpath, ACL_TYPE_DEFAULT);
     if ((acl_a == NULL) && (errno == ENOTSUP)) {
       line->attr&=(~DB_ACL);
       return;
     }
     if (acl_a == NULL)
-      error(0, "Tried to read access ACL on %s but failed with: %m\n",
-            line->filename);
+      error(0, "Tried to read access ACL on %s but failed with: %s\n",
+            line->fullpath, strerror(errno));
     if ((acl_d == NULL) && (errno != EACCES)) /* ignore DEFAULT on files */
     {
       acl_free(acl_a);
-      error(0, "Tried to read default ACL on %s but failed with: %m\n",
-            line->filename);
+      error(0, "Tried to read default ACL on %s but failed with: %s\n",
+            line->fullpath, strerror(errno));
     }
 
     /* assume memory allocs work, like rest of AIDE code... */
@@ -509,18 +507,18 @@ void acl2line(db_line* line) {
   if(DB_ACL&line->attr) { /* There might be a bug here. */
     int res;
     line->acl=malloc(sizeof(acl_type));
-    line->acl->entries=acl(line->filename,GETACLCNT,0,NULL);
+    line->acl->entries=acl(line->fullpath,GETACLCNT,0,NULL);
     if (line->acl->entries==-1) {
       char* er=strerror(errno);
       line->acl->entries=0;
       if (er==NULL) {
-	error(0,"ACL query failed for %s. strerror failed for %i\n",line->filename,errno);
+	error(0,"ACL query failed for %s. strerror failed for %i\n",line->fullpath,errno);
       } else {
-	error(0,"ACL query failed for %s:%s\n",line->filename,er);
+	error(0,"ACL query failed for %s:%s\n",line->fullpath,er);
       }
     } else {
       line->acl->acl=malloc(sizeof(aclent_t)*line->acl->entries);
-      res=acl(line->filename,GETACL,line->acl->entries,line->acl->acl);
+      res=acl(line->fullpath,GETACL,line->acl->entries,line->acl->acl);
       if (res==-1) {
 	error(0,"ACL error %s\n",strerror(errno));
       } else {
@@ -534,12 +532,130 @@ void acl2line(db_line* line) {
   }
 #endif
 }
+#endif
+
+#ifdef WITH_XATTR
+static xattrs_type *xattr_new(void) {
+    xattrs_type *ret = NULL;
+
+    ret = malloc(sizeof(xattrs_type));
+    ret->num = 0;
+    ret->sz  = 2;
+    ret->ents = malloc(sizeof(xattr_node) * ret->sz);
+
+    return (ret);
+}
+
+static void *xzmemdup(const void *ptr, size_t len) {
+    /* always keeps a 0 at the end... */
+    void *ret = NULL;
+
+    ret = malloc(len+1);
+    memcpy(ret, ptr, len);
+    ((char*)ret)[len] = 0;
+
+    return (ret);
+}
+
+static void xattr_add(xattrs_type *xattrs, const char *key, const char
+        *val, size_t vsz) {
+    if (xattrs->num >= xattrs->sz) {
+        xattrs->sz <<= 1;
+        xattrs->ents = realloc(xattrs->ents, sizeof(xattr_node) * xattrs->sz);
+    }
+
+    xattrs->ents[xattrs->num].key = strdup(key);
+    xattrs->ents[xattrs->num].val = xzmemdup(val, vsz);
+    xattrs->ents[xattrs->num].vsz = vsz;
+
+    xattrs->num += 1;
+}
+
+void xattrs2line(db_line *line) {
+    /* get all generic user xattrs. */
+    xattrs_type *xattrs = NULL;
+    static ssize_t xsz = 1024;
+    static char *xatrs = NULL;
+    ssize_t xret = -1;
+
+    if (!(DB_XATTRS&line->attr))
+        return;
+
+    /* assume memory allocs work, like rest of AIDE code... */
+    if (!xatrs) xatrs = malloc(xsz);
+
+    while (((xret = llistxattr(line->fullpath, xatrs, xsz)) == -1) && (errno == ERANGE)) {
+        xsz <<= 1;
+        xatrs = realloc(xatrs, xsz);
+    }
+
+    if ((xret == -1) && ((errno == ENOSYS) || (errno == ENOTSUP))) {
+        line->attr&=(~DB_XATTRS);
+    } else if (xret == -1) {
+        error(0, "listxattrs failed for %s:%s\n", line->fullpath, strerror(errno));
+    } else if (xret) {
+        const char *attr = xatrs;
+        static ssize_t asz = 1024;
+        static char *val = NULL;
+
+        if (!val) val = malloc(asz);
+
+        xattrs = xattr_new();
+
+        while (xret > 0) {
+            size_t len = strlen(attr);
+            ssize_t aret = 0;
+
+            if (strncmp(attr, "user.", strlen("user.")) &&
+                    strncmp(attr, "root.", strlen("root.")))
+                goto next_attr; /* only store normal xattrs, and SELinux */
+
+            while (((aret = getxattr(line->fullpath, attr, val, asz)) ==
+                        -1) && (errno == ERANGE)) {
+                asz <<= 1;
+                val = realloc (val, asz);
+            }
+
+            if (aret != -1)
+                xattr_add(xattrs, attr, val, aret);
+            else if (errno != ENOATTR)
+                error(0, "getxattr failed for %s:%s\n", line->fullpath, strerror(errno));
+
+next_attr:
+            attr += len + 1;
+            xret -= len + 1;
+        }
+    }
+
+    line->xattrs = xattrs;
+}
+#endif
+
+#ifdef WITH_SELINUX
+void selinux2line(db_line *line) {
+    char *cntx = NULL;
+
+    if (!(DB_SELINUX&line->attr))
+        return;
+
+    if (lgetfilecon_raw(line->fullpath, &cntx) == -1) {
+        line->attr&=(~DB_SELINUX);
+        if ((errno != ENOATTR) && (errno != EOPNOTSUPP))
+            error(0, "lgetfilecon_raw failed for %s:%s\n", line->fullpath, strerror(errno));
+        return;
+    }
+
+    line->cntx = strdup(cntx);
+
+    freecon(cntx);
+}
+#endif
 
 #ifdef WITH_E2FSATTRS
 void e2fsattrs2line(db_line* line) {
     unsigned long flags;
     if (DB_E2FSATTRS&line->attr) {
-        if (fgetflags(line->filename, &flags) == 0) {
+        if (fgetflags(line->fullpath, &flags) == 0) {
             line->e2fsattrs=flags;
         } else {
             line->attr&=(~DB_E2FSATTRS);
